<?php

namespace TC\PaygateHeleket\Pub\Controller;

use XF;
use XF\Entity\PurchaseRequest;
use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class Heleket extends AbstractController
{
	/**
	 * @param ParameterBag $params
	 *
	 * @return XF\Mvc\Reply\AbstractReply
	 */
	public function actionStatus(ParameterBag $params): XF\Mvc\Reply\AbstractReply
	{
		$purchaseRequest = $this->assertPurchaseRequest($params->request_key ?? '');
		$purchase = $this->assertPurchaseObject($purchaseRequest);
		$payment = $this->getPayment($purchaseRequest, $error);

		if ($error || empty($payment['address']))
		{
			return $this->error(XF::phrase('tc_pg_heleket_errors_payment_not_found'));
		}

		if (in_array($payment['status'], [
			'paid',
			'paid_over'
		]))
		{
			return $this->redirect(
				$purchaseRequest->provider_metadata == 'completed'
					? $purchase->returnUrl
					: $purchase->cancelUrl
			);
		}

		if ($payment['is_final'] && $payment['expired_at'] <= XF::$time)
		{
			return $this->error(XF::phrase('tc_pg_heleket_errors_invoice_expired'));
		}

		$viewParams = [
			'invoice'         => $payment,
			'purchaseRequest'  => $purchaseRequest,
			'purchase'         => $purchase,
		];

		return $this->view(
			'TC\PaygateHeleket:Payment\Status',
			'tc_payment_process_heleket',
			$viewParams
		);
	}

	public function actionStatusJson(ParameterBag $params)
	{
		$purchaseRequest = $this->assertPurchaseRequest($params->request_key ?? '');
		$payment = $this->getPayment($purchaseRequest, $error);

		$viewParams = [
			'payment' => $payment
		];

		return $this->view('TC\PaygateHeleket:Payment\Json', '', $viewParams);
	}

	protected function getPayment(PurchaseRequest $purchaseRequest, &$error = null)
	{
		if ($purchaseRequest->user_id != XF::visitor()->user_id || empty($purchaseRequest->provider_metadata))
		{
			throw $this->exception($this->error(XF::phrase('invalid_purchase_request')));
		}

		$paymentProfile = $purchaseRequest->PaymentProfile;
		$profileOptions = $paymentProfile->options;

		/** @var \TC\PaygateHeleket\Payment\Heleket $handler */
		$handler = $paymentProfile->getPaymentHandler();

		// Payment already created. Fetch info from Heleket
		$payment = $handler->apiCall($profileOptions['merchant_id'], $profileOptions['api_key'], 'v1/payment/info',
			[
				'uuid' => $purchaseRequest->provider_metadata
			],
			$error
		);

		if($error)
		{
			throw $this->exception($this->error($error));
		}

		$payment['payment_link'] = $handler->generatePaymentLink($payment['network'], $payment['address'], $payment['payer_amount']);

		return $payment;
	}

	/**
	 * @param string $requestKey
	 *
	 * @return PurchaseRequest
	 * @throws XF\Mvc\Reply\Exception
	 */
	protected function assertPurchaseRequest(string $requestKey): PurchaseRequest
	{
		/** @var PurchaseRequest $purchaseRequest */
		$purchaseRequest = $this->em()->findOne(PurchaseRequest::class, [
			'request_key' => $requestKey,
			'provider_id' => 'tcHeleket'
		], ['User', 'PaymentProfile']);

		if (!$purchaseRequest || $purchaseRequest->user_id != XF::visitor()->user_id || empty($purchaseRequest->provider_metadata))
		{
			throw $this->exception($this->error(XF::phrase('invalid_purchase_request')));
		}

		return $purchaseRequest;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 *
	 * @return XF\Purchasable\Purchase
	 * @throws XF\Mvc\Reply\Exception
	 */
	protected function assertPurchaseObject(PurchaseRequest $purchaseRequest): XF\Purchasable\Purchase
	{
		$purchasableHandler = $purchaseRequest->Purchasable->getHandler();
		$purchasable = $purchasableHandler->getPurchasableFromExtraData($purchaseRequest->extra_data)['purchasable'] ?? [];

		if (empty($purchasable))
		{
			throw $this->exception($this->error(XF::phrase('invalid_purchase_request')));
		}

		return $purchasableHandler->getPurchaseObject(
			$purchaseRequest->PaymentProfile,
			$purchasable,
			$purchaseRequest->User
		);
	}
}
